#include "globals.h"
#include <pwd.h>
#include <grp.h>

static Boolean isPoppedUp = False;
static Widget propertiesShell = NULL, propertiesTopForm, userReadTB, 
		groupReadTB, otherReadTB, userWriteTB,
		groupWriteTB, otherWriteTB, userExecTB, groupExecTB,
		otherExecTB, propertiesFilenameVal, propertiesSizeVal,
		propertiesLinkCtVal, propertiesOwnerVal, propertiesGroupVal,
		propertiesLastModVal;
	      

void propertiesOKCB( 
#if NeedFunctionPrototypes
	Widget, XtPointer, XtPointer 
#endif
);
void propertiesCancelCB( 
#if NeedFunctionPrototypes
	Widget, XtPointer, XtPointer 
#endif
);
int comparePermissions( 
#if NeedFunctionPrototypes
	mode_t, mode_t 
#endif
);


void propertiesCB( w, xtp1, xtp2 )
  Widget w;
  XtPointer xtp1, xtp2;
{
  struct FileManager *f = (struct FileManager *)xtp1;
  XmString xms;
  char tmpString[100];
  int i;
  struct passwd *pw_entry;
  struct group *grp_entry;
  Boolean state;
  unsigned short permissions;
  Atom wmDel;
  struct DirInfo *dim1, *dim2;
  struct FileList *fl;

  if( f->nDirInfo < 2 )  return;

  dim2 = getIthDir( f->dirInfo, f->nDirInfo-2 );
  dim1 = getIthDir( f->dirInfo, f->nDirInfo-1 );
  for( i = 0; i < dim2->nFiles; i++ )
  {
    fl = getIthFile( dim2->fileList, i );
    if( !strcmp( dim1->di.dirName, fl->name ) )  break;
  }

  if( i >= dim2->nFiles )  return;

  if( !propertiesShell )
  {
    XmString xms;
    Widget propertiesForm, propertiesFilenameLBL, propertiesSizeLBL,
	   propertiesLinkCtLBL, propertiesOwnerLBL, propertiesGroupLBL,
	   propertiesLastModLBL, propertiesPermLBL, permRC,
	   permUserLBL, permGroupLBL, permOtherLBL, permReadLBL,
	   permWriteLBL, permExecLBL,
	   separator, okButton, cancelButton;

    propertiesShell = XtVaCreatePopupShell( "Properties",
		transientShellWidgetClass, w,
		XmNallowShellResize, False,
		NULL );

    xms = XmStringCreateSimple( "" );

    propertiesTopForm = XtVaCreateManagedWidget( "propertiesTopForm",
		xmFormWidgetClass, propertiesShell, NULL );

    XtVaSetValues( propertiesShell, XmNdeleteResponse, XmDO_NOTHING, NULL );
    wmDel = XmInternAtom( XtDisplay(propertiesTopForm), "WM_DELETE_WINDOW", False );
    XmAddWMProtocolCallback( propertiesShell, wmDel, propertiesCancelCB, propertiesTopForm );

    separator = XtVaCreateManagedWidget( "separator",
		xmSeparatorWidgetClass, propertiesTopForm,
		XmNbottomAttachment, XmATTACH_FORM,
                XmNbottomOffset, 60,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		NULL );

    okButton = XtVaCreateManagedWidget( "Ok",
		xmPushButtonWidgetClass, propertiesTopForm,
		XmNbottomAttachment, XmATTACH_FORM,
		XmNbottomOffset, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 15,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 35,
		XmNheight, 30,
		XmNshowAsDefault, True,
                NULL );
    XtAddCallback( okButton, XmNactivateCallback, propertiesOKCB, (XtPointer)f );

    cancelButton = XtVaCreateManagedWidget( "Cancel",
                xmPushButtonWidgetClass, propertiesTopForm,
		XmNbottomAttachment, XmATTACH_FORM,
                XmNbottomOffset, 15,
                XmNleftAttachment, XmATTACH_POSITION,
                XmNleftPosition, 65,
                XmNrightAttachment, XmATTACH_POSITION,
                XmNrightPosition, 85,
                XmNheight, 30,
                NULL );
    XtAddCallback( cancelButton, XmNactivateCallback, propertiesCancelCB, NULL );

    propertiesForm = XtVaCreateManagedWidget( "propertiesForm",
		xmFormWidgetClass, propertiesTopForm, 
		XmNtopAttachment, XmATTACH_FORM,
		XmNtopOffset, 10,
		XmNleftAttachment, XmATTACH_FORM,
		XmNleftOffset, 10,
		XmNrightAttachment, XmATTACH_FORM,
		XmNrightOffset, 10,
		XmNbottomAttachment, XmATTACH_FORM,
		XmNbottomOffset, 50,
		NULL );

    propertiesFilenameLBL = XtVaCreateManagedWidget( "File Name:",
		xmLabelWidgetClass, propertiesForm, 
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		XmNleftOffset, 10,
		XmNheight, 25,
		NULL );
    propertiesFilenameVal = XtVaCreateManagedWidget( "propertiesFilenameVal",
		xmTextFieldWidgetClass, propertiesForm,
		XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET,
		XmNtopWidget, propertiesFilenameLBL,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, propertiesFilenameLBL,
		XmNrightAttachment, XmATTACH_FORM,
		XmNrightOffset, 10,
		XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET,
		XmNbottomWidget, propertiesFilenameLBL,
		XmNmarginHeight, 0,
		XmNeditable, False,
		NULL );
    propertiesSizeLBL = XtVaCreateManagedWidget( "Size(bytes):",
		xmLabelWidgetClass, propertiesForm,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopOffset, 10,
		XmNtopWidget, propertiesFilenameLBL,
		XmNleftAttachment, XmATTACH_FORM,
		XmNleftOffset, 10,
		XmNheight, 25,
		XmNborderWidth, 1,
		NULL );
    propertiesSizeVal = XtVaCreateManagedWidget( "propertiesSizeVal",
		xmLabelWidgetClass, propertiesForm,
		XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET,
		XmNtopWidget, propertiesSizeLBL,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, propertiesSizeLBL,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 50,
		XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET,
                XmNbottomWidget, propertiesSizeLBL,
		XmNlabelString, xms,
		XmNresizable, False,
		NULL );
    propertiesLinkCtLBL = XtVaCreateManagedWidget( "Link Count:",
		xmLabelWidgetClass, propertiesForm,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopOffset, 10,
                XmNtopWidget, propertiesFilenameLBL,
                XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 50,
		XmNheight, 25,
		XmNborderWidth, 1,
		NULL );
    propertiesLinkCtVal = XtVaCreateManagedWidget( "propertiesLinkCtVal",
		xmLabelWidgetClass, propertiesForm,
		XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET,
                XmNtopWidget, propertiesLinkCtLBL,
                XmNleftAttachment, XmATTACH_WIDGET,
                XmNleftWidget, propertiesLinkCtLBL,
		XmNrightAttachment, XmATTACH_FORM,
		XmNrightOffset, 10,
		XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET,
                XmNbottomWidget, propertiesLinkCtLBL,
                XmNlabelString, xms,
		XmNresizable, False,
                NULL );
    propertiesOwnerLBL = XtVaCreateManagedWidget( "Owner:",
		xmLabelWidgetClass, propertiesForm,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, propertiesSizeLBL,
		XmNleftAttachment, XmATTACH_FORM,
		XmNleftOffset, 10,
		XmNheight, 25,
		XmNborderWidth, 1,
		NULL );
    propertiesOwnerVal = XtVaCreateManagedWidget( "propertiesOwnerVal",
		xmLabelWidgetClass, propertiesForm,
		XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET,
                XmNtopWidget, propertiesOwnerLBL,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, propertiesOwnerLBL,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 50,
		XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET,
                XmNbottomWidget, propertiesOwnerLBL,
                XmNlabelString, xms,
		XmNresizable, False,
                NULL );
    propertiesGroupLBL = XtVaCreateManagedWidget( "Group:",
		xmLabelWidgetClass, propertiesForm,
                XmNtopAttachment, XmATTACH_WIDGET,
                XmNtopWidget, propertiesSizeLBL,
                XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 50,
		XmNheight, 25,
		XmNborderWidth, 1,
		NULL );
    propertiesGroupVal = XtVaCreateManagedWidget( "propertiesGroupVal",
		xmLabelWidgetClass, propertiesForm,
		XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET,
                XmNtopWidget, propertiesGroupLBL,
                XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, propertiesGroupLBL,
		XmNrightAttachment, XmATTACH_FORM,
		XmNrightOffset, 10,
		XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET,
                XmNbottomWidget, propertiesGroupLBL,
		XmNresizable, False,
		XmNlabelString, xms,
                NULL );
    propertiesLastModLBL = XtVaCreateManagedWidget( "Last Modified:",
		xmLabelWidgetClass, propertiesForm,
		XmNtopAttachment, XmATTACH_WIDGET,
                XmNtopWidget, propertiesOwnerLBL,
		XmNleftAttachment, XmATTACH_FORM,
		XmNleftOffset, 10,
		XmNheight, 25,
		XmNborderWidth, 1,
		NULL );
    propertiesLastModVal = XtVaCreateManagedWidget( "propertiesLastModVal",
		xmLabelWidgetClass, propertiesForm,
		XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET,
                XmNtopWidget, propertiesLastModLBL,
                XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, propertiesLastModLBL,
		XmNleftOffset, 10,
		XmNrightAttachment, XmATTACH_FORM,
		XmNrightOffset, 10,
		XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET,
                XmNbottomWidget, propertiesLastModLBL,
                XmNresizable, False,
                XmNlabelString, xms,
		XmNalignment, XmALIGNMENT_BEGINNING,
                NULL );
    permRC = XtVaCreateManagedWidget( "permRC",
		xmRowColumnWidgetClass, propertiesForm,
		XmNtopAttachment, XmATTACH_WIDGET,
                XmNtopWidget, propertiesLastModLBL,
		XmNtopOffset, 10,
		XmNleftAttachment, XmATTACH_FORM,
                XmNleftOffset, 10,
		XmNbottomAttachment, XmATTACH_WIDGET,
		XmNbottomWidget, separator,
		XmNbottomOffset, 10,
		XmNpacking, XmPACK_COLUMN,
		XmNorientation, XmHORIZONTAL,
		XmNnumColumns, 4,
		NULL );
    propertiesPermLBL = XtVaCreateManagedWidget( "Permissions:",
		xmLabelWidgetClass, permRC,
		NULL );
    permUserLBL = XtVaCreateManagedWidget( "User",
		xmLabelWidgetClass, permRC,
                NULL );
    permGroupLBL = XtVaCreateManagedWidget( "Group",
                xmLabelWidgetClass, permRC,
                NULL );
    permOtherLBL = XtVaCreateManagedWidget( "Other",
                xmLabelWidgetClass, permRC,
                NULL );
    permReadLBL = XtVaCreateManagedWidget( "Read",
                xmLabelWidgetClass, permRC,
                NULL );
    userReadTB = XtVaCreateManagedWidget( "userReadTB",
		xmToggleButtonWidgetClass, permRC,
		XmNlabelString, xms,
		XmNtraversalOn, False,
		NULL );
    groupReadTB = XtVaCreateManagedWidget( "groupReadTB",
		xmToggleButtonWidgetClass, permRC,
                XmNlabelString, xms,
		XmNtraversalOn, False,
                NULL );
    otherReadTB = XtVaCreateManagedWidget( "otherReadTB",
		xmToggleButtonWidgetClass, permRC,
                XmNlabelString, xms,
		XmNtraversalOn, False,
                NULL );
    permWriteLBL = XtVaCreateManagedWidget( "Write",
                xmLabelWidgetClass, permRC,
                NULL );
    userWriteTB = XtVaCreateManagedWidget( "userWriteTB",
		xmToggleButtonWidgetClass, permRC,
                XmNlabelString, xms,
		XmNtraversalOn, False,
                NULL );
    groupWriteTB = XtVaCreateManagedWidget( "groupWriteTB",
		xmToggleButtonWidgetClass, permRC,
                XmNlabelString, xms,
		XmNtraversalOn, False,
                NULL );
    otherWriteTB = XtVaCreateManagedWidget( "otherWriteTB",
		xmToggleButtonWidgetClass, permRC,
                XmNlabelString, xms,
		XmNtraversalOn, False,
                NULL );
    permExecLBL = XtVaCreateManagedWidget( "Exec",
                xmLabelWidgetClass, permRC,
                NULL );
    userExecTB = XtVaCreateManagedWidget( "userExecTB",
		xmToggleButtonWidgetClass, permRC,
                XmNlabelString, xms,
		XmNtraversalOn, False,
                NULL );
    groupExecTB = XtVaCreateManagedWidget( "groupExecTB",
		xmToggleButtonWidgetClass, permRC,
		XmNlabelString, xms,
		XmNtraversalOn, False,
                NULL );
    otherExecTB = XtVaCreateManagedWidget( "otherExecTB",
		xmToggleButtonWidgetClass, permRC,
		XmNlabelString, xms,
		XmNtraversalOn, False,
                NULL );

    XmStringFree( xms );
  }

  XtVaSetValues( propertiesTopForm, XmNuserData, (XtPointer)fl, NULL );

  sprintf( tmpString, "%d", fl->s.st_size );
  xms = XmStringCreateSimple( tmpString );
  XtVaSetValues( propertiesSizeVal, XmNlabelString, xms, NULL );
  XmStringFree( xms );

  sprintf( tmpString, "%d", fl->s.st_nlink );
  xms = XmStringCreateSimple( tmpString );
  XtVaSetValues( propertiesLinkCtVal, XmNlabelString, xms, NULL );
  XmStringFree( xms );

  pw_entry = getpwuid( fl->s.st_uid );
  xms = XmStringCreateSimple( pw_entry->pw_name );
  XtVaSetValues( propertiesOwnerVal, XmNlabelString, xms, NULL );
  XmStringFree( xms );

  grp_entry = getgrgid( fl->s.st_gid );
  xms = XmStringCreateSimple( grp_entry->gr_name );
  XtVaSetValues( propertiesGroupVal, XmNlabelString, xms, NULL );
  XmStringFree( xms );

  strftime( tmpString, sizeof(tmpString), "%c", localtime( &fl->s.st_mtime ) );
  xms = XmStringCreateSimple( tmpString );
  XtVaSetValues( propertiesLastModVal, XmNlabelString, xms, NULL );
  XmStringFree( xms );

  state = ( !strcmp( cuserid(NULL), pw_entry->pw_name ) ||
            !strcmp( "root", cuserid(NULL) ) ) ? True : False;

  XtVaSetValues( userReadTB, XmNsensitive, state, NULL );
  XtVaSetValues( groupReadTB, XmNsensitive, state, NULL );
  XtVaSetValues( otherReadTB, XmNsensitive, state, NULL );
  XtVaSetValues( userWriteTB, XmNsensitive, state, NULL );
  XtVaSetValues( groupWriteTB, XmNsensitive, state, NULL );
  XtVaSetValues( otherWriteTB, XmNsensitive, state, NULL );
  XtVaSetValues( userExecTB, XmNsensitive, state, NULL );
  XtVaSetValues( groupExecTB, XmNsensitive, state, NULL );
  XtVaSetValues( otherExecTB, XmNsensitive, state, NULL );

  XmTextFieldSetString( propertiesFilenameVal, fl->name );
  XtVaSetValues( propertiesFilenameVal, XmNeditable, state, NULL );

  permissions = fl->s.st_mode;

  state = (permissions & S_IRUSR) ? True : False;
  XtVaSetValues( userReadTB, XmNset, state, NULL );
  state = (permissions & S_IWUSR) ? True : False;
  XtVaSetValues( userWriteTB, XmNset, state, NULL );
  state = (permissions & S_IXUSR) ? True : False;
  XtVaSetValues( userExecTB, XmNset, state, NULL );
  state = (permissions & S_IRGRP) ? True : False;
  XtVaSetValues( groupReadTB, XmNset, state, NULL );
  state = (permissions & S_IWGRP) ? True : False;
  XtVaSetValues( groupWriteTB, XmNset, state, NULL );
  state = (permissions & S_IXGRP) ? True : False;
  XtVaSetValues( groupExecTB, XmNset, state, NULL );
  state = (permissions & S_IROTH) ? True : False;
  XtVaSetValues( otherReadTB, XmNset, state, NULL );
  state = (permissions & S_IWOTH) ? True : False;
  XtVaSetValues( otherWriteTB, XmNset, state, NULL );
  state = (permissions & S_IXOTH) ? True : False;
  XtVaSetValues( otherExecTB, XmNset, state, NULL );

  isPoppedUp = True;
  XtPopup( propertiesShell, XtGrabNone );
  XRaiseWindow( XtDisplay(propertiesShell), XtWindow(propertiesShell) );
}


/*  callbacks  */

void propertiesOKCB( w, xtp1, xtp2 )
  Widget w;
  XtPointer xtp1, xtp2;
{
  struct FileManager *f = (struct FileManager *)xtp1;
  struct DirInfo *dim2;
  struct FileList *dp;
  mode_t permissions;
  Boolean set;
  Arg args[1];
  int *pos_list, pos_count=0, *pos_list2, pos_count2=0;
  char *fileName, *newFileName;
  
  XtVaGetValues( propertiesTopForm, XmNuserData, (XtArgVal)(&dp), NULL );

  XtSetArg( args[0], XmNset, &set );
  permissions = 0;
  
  XtGetValues( userReadTB, args, 1 );
  if( set )  permissions |= S_IRUSR;
  XtGetValues( userWriteTB, args, 1 );
  if( set )  permissions |= S_IWUSR;
  XtGetValues( userExecTB, args, 1 );
  if( set )  permissions |= S_IXUSR;
  XtGetValues( groupReadTB, args, 1 );
  if( set )  permissions |= S_IRGRP;
  XtGetValues( groupWriteTB, args, 1 );
  if( set )  permissions |= S_IWGRP;
  XtGetValues( groupExecTB, args, 1 );
  if( set )  permissions |= S_IXGRP;
  XtGetValues( otherReadTB, args, 1 );
  if( set )  permissions |= S_IROTH;
  XtGetValues( otherWriteTB, args, 1 );
  if( set )  permissions |= S_IWOTH;
  XtGetValues( otherExecTB, args, 1 );
  if( set )  permissions |= S_IXOTH;

  newFileName = XmTextFieldGetString( propertiesFilenameVal );

  /*  if it isn't different, return  */
  if( comparePermissions( permissions, dp->s.st_mode ) && 
      !strcmp( newFileName, dp->name ) )  
  {
    XtFree( (char *)newFileName );
    return;
  }

  dim2 = getIthDir( f->dirInfo, f->nDirInfo-2 );
  fileName = (char *)XtMalloc( strlen(dim2->di.dragName) + strlen(dp->name) + 4 );
  strcpy( fileName, dim2->di.dragName );
  strcat( fileName, "/" );
  strcat( fileName, dp->name );

  if( permissions != dp->s.st_mode )
  {
    if( chmod( fileName, permissions) == -1 )
    {
      char errorString[100];
      sprintf( errorString, "***ERROR: Failed to change permissions on:\n%s", fileName );
      popupErrorDialog( errorString );
      XtFree( (char *)newFileName );
      XtFree( (char *)fileName );
      return;
    }
  }

  if( strcmp( newFileName, dp->name ) )
  {
    char *tmpStr;
    tmpStr = (char *)XtMalloc( strlen(dim2->di.dragName) + 3 + strlen(newFileName) );
    strcpy( tmpStr, dim2->di.dragName );
    strcat( tmpStr, "/" );
    strcat( tmpStr, newFileName );

    if( rename( fileName, tmpStr ) == -1 )
    {
      char errorString[200];
      sprintf( errorString, "***ERROR: Failed to rename %s to %s\n", fileName, tmpStr );
      popupErrorDialog( errorString );
      XtFree( (char *)tmpStr );
      return;
    }

    XtFree( (char *)tmpStr );
  }

  XtFree( (char *)fileName );
  XtFree( (char *)newFileName );

  updateAllFileManagers();
}


void propertiesCancelCB( w, xtp1, xtp2 )
  Widget w;
  XtPointer xtp1, xtp2;
{
  isPoppedUp = False;
  XtPopdown( propertiesShell );
}

int comparePermissions( p1, p2 )
  mode_t p1, p2;
{
  if( ( (p1 & S_IRUSR) == (p2 & S_IRUSR) ) &&
      ( (p1 & S_IWUSR) == (p2 & S_IWUSR) ) &&
      ( (p1 & S_IXUSR) == (p2 & S_IXUSR) ) &&
      ( (p1 & S_IRGRP) == (p2 & S_IRGRP) ) &&
      ( (p1 & S_IWGRP) == (p2 & S_IWGRP) ) && 
      ( (p1 & S_IXGRP) == (p2 & S_IXGRP) ) && 
      ( (p1 & S_IROTH) == (p2 & S_IROTH) ) && 
      ( (p1 & S_IWOTH) == (p2 & S_IWOTH) ) && 
      ( (p1 & S_IXOTH) == (p2 & S_IXOTH) ) )
  {
    return 1;
  }
  else
  {
    return 0;
  }
}
